/** Code is released under GPLv2
 *
 * See debian/copyright for copyright info
 */

/** Compile binary only with:
 * gcc -Wall `pkg-config --cflags --libs hildon-notify glib-2.0 mafw mafw-shared` nowplaying.c -o nowplayingd
 */

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <libintl.h>
#include <locale.h>
#include <hildon/hildon-notification.h>
#include <libmafw/mafw.h>
#include <libmafw-shared/mafw-shared.h>

#define WANTED_RENDERER "Mafw-Gst-Renderer"

static GMainLoop *loop = NULL;
static MafwRegistry *registry = NULL;

static void sig_handler( int sig G_GNUC_UNUSED )
{
	if ( g_main_loop_is_running( loop ) ) {
		g_main_loop_quit( loop );
	}
}

static gchar* filename_from_object_id(
	const gchar *object_id)
{
	/* TODO: Strip off the extension */
	g_return_val_if_fail (object_id, NULL);

	gchar *path_uri = g_filename_display_basename( object_id ); /* Remove "localtagfs::music/songs". Pointer arith. would probably work, but I don't wan't to hardcode anything. */
	gchar *path_unescaped = g_uri_unescape_string( path_uri, NULL ); /* Make the string look normal */
	gchar *ret_filename = g_filename_display_basename( path_unescaped ); /* Get the filename w/out path */

	g_free (path_uri);
	g_free (path_unescaped);

	return ret_filename ? ret_filename : NULL;
}

static void metadata_callback(
	const MafwRenderer *self G_GNUC_UNUSED,
	const gchar *object_id,
	GHashTable *metadata,
	gconstpointer user_data G_GNUC_UNUSED,
	const GError *error G_GNUC_UNUSED )
{
	gchar *title = NULL;
	const gchar *artist = NULL;
	HildonNotification *n = NULL;

	/* First: Try to get title and artist strings from tags: */
	title = g_value_dup_string( mafw_metadata_first ( metadata, MAFW_METADATA_KEY_TITLE ) );
	artist = g_value_get_string( mafw_metadata_first ( metadata, MAFW_METADATA_KEY_ARTIST ) );

	/* Secondly: If getting the title from the tags fails, fallback to the filename, instead: */
	if ( !title || *title == '\0' ) /* || !strcmp (title, "")) */ {
		title = filename_from_object_id ( object_id );
	}

	/* Thirdly: If above fails, we just resort to "(unknown song)" */
	if ( !title || *title == '\0' ) /* || !strcmp (title, "")) */ {
		title = g_strdup( dgettext( "mediaplayer", "mp_li_unknown_song" ) );
	}

	if ( !artist || *artist == '\0' ) /* || !strcmp (artist, "")) */ {
		artist = g_dgettext( "mediaplayer", "mp_li_unknown_artist" );
	}

	n = hildon_notification_new( artist, title, "tasklaunch_media_player", "media" );
	hildon_notification_add_dbus_action( n, "default",
		"tasklaunch_media_player",
		"com.nokia.mediaplayer",
		"/com/nokia/osso/mediaplayer",
		"com.nokia.mediaplayer",
		"open_mp_now_playing",
		G_TYPE_NONE, NULL,
		-1 );

	notify_notification_set_timeout( NOTIFY_NOTIFICATION( n ), 4500 /* 4.5s */ );
	notify_notification_show( NOTIFY_NOTIFICATION( n ), NULL );

	g_free ( title );
}

static void state_changed_cb(
	MafwRenderer *renderer,
	const MafwPlayState state,
	gconstpointer user_data G_GNUC_UNUSED)
{
	switch (state) {
		case Playing:
			mafw_renderer_get_current_metadata( renderer,
				(MafwRendererMetadataResultCB) metadata_callback,
				NULL );
			break;

		case Stopped:
		default:
			break;
  }
}

static void renderer_removed_cb(
	const MafwRegistry *registry2 G_GNUC_UNUSED,
	GObject *renderer,
	gconstpointer user_data G_GNUC_UNUSED)
{
	if ( MAFW_IS_RENDERER( renderer ) ) {
		const gchar *name = mafw_extension_get_name( MAFW_EXTENSION( renderer ) );

		if ( !strcmp( name, WANTED_RENDERER ) ) {
			g_signal_handlers_disconnect_by_func( renderer, G_CALLBACK( state_changed_cb ), NULL );
		}
	}
}

static void renderer_added_cb(
	const MafwRegistry *registry2 G_GNUC_UNUSED,
	GObject *renderer,
	gconstpointer user_data G_GNUC_UNUSED)
{
	if ( MAFW_IS_RENDERER( renderer ) ) {
		const gchar *name = mafw_extension_get_name( MAFW_EXTENSION( renderer ) );

		if ( !strcmp( name, WANTED_RENDERER ) ) {
			g_signal_connect( renderer, "state-changed", G_CALLBACK (state_changed_cb), NULL );
		}
	}
}

static void mafw_deinit( void )
{
    GList *renderers = NULL;

	g_signal_handlers_disconnect_by_func( registry, G_CALLBACK( renderer_removed_cb ), NULL );
	renderers = mafw_registry_get_renderers( registry );
	while ( renderers ) {
		renderer_removed_cb( registry, G_OBJECT( renderers->data ), NULL );
		renderers = g_list_next( renderers );
	}
}

static void mafw_init( void )
{
    GList *renderers = NULL;

	g_signal_connect( registry, "renderer-added", G_CALLBACK( renderer_added_cb ), NULL );
	renderers = mafw_registry_get_renderers( registry );
	while ( renderers ) {
		renderer_added_cb( registry, G_OBJECT( renderers->data ), NULL );
		renderers = g_list_next( renderers );
	}
}

int main (void)
{
	setlocale( LC_ALL, "" );

	g_type_init();
	notify_init("nowplayingd");

	loop = g_main_loop_new( NULL, FALSE );
	g_assert( loop );

	registry = MAFW_REGISTRY( mafw_registry_get_instance() );
	g_assert( registry );

	signal( SIGINT, sig_handler );
	signal( SIGQUIT, sig_handler );
	signal( SIGTERM, sig_handler );

	mafw_shared_init( registry, NULL );

	mafw_init();

	g_main_loop_run( loop );

	g_main_loop_unref( loop );

	mafw_deinit();

	mafw_shared_deinit();

	notify_uninit();

	return EXIT_SUCCESS;
}

